<?php
// Set up vars to track results.
$errorMessages = ''; // Composite error messsage.
$taxRate = 0; // Tax rate for a state.
$stateShortName = '';
$stateLongName = '';
$price = 0; // List price.
$discountRate = 0; // Discount rate, e.g., 0.1
$discountedPrice = 0; // Price after discount taken.
$tax = 0; // Tax, price * rate.
$total = 0; // Total after tax.
// Check the state.
$stateShortName = getParamFromGet('state');
$stateErrorMessage = checkState($stateShortName);
// Check the price.
$price = getParamFromGet('price');
$priceErrorMessage = checkPrice($price);
// Check the discount rate.
$discountRate = getParamFromGet('discount_rate');
$discountRateErrorMessage = checkDiscountRate($discountRate);
// Combine the error messages.
$errorMessages = $stateErrorMessage . $priceErrorMessage . $discountRateErrorMessage;
// Should there be processing?
if ($errorMessages == '') {
    // No errors.
    if ($stateShortName == 'mi') {
        $taxRate = 0.06;
        $stateLongName = 'Michigan';
    }
    else {
        $taxRate = 0.0625;;
        $stateLongName = 'Illinois';
    }
    $discount = $price * $discountRate;
    $discountedPrice = $price - $discount;
    $tax = $discountedPrice * $taxRate;
    $total = $discountedPrice + $tax;
}
?><!doctype html>
<html lang="en">
    <head>
        <title>Sales tax and discount</title>
        <meta charset="utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
        <link rel="stylesheet" href="styles.css">
    </head>
    <body>
        <h1>Sales tax and discount</h1>
        <?php
        if ($errorMessages != '') {
            print "<div class='error-message'>
                     <p>There were errors.</p>
                     <p>$errorMessages</p>
                   </div>\n";
        }
        else {
            print "<p>List price: $price</p>
                   <p>Discount rate: $discountRate</p>
                   <p>Discounted price: $discountedPrice</p> 
                   <p>Sales tax ($stateLongName, $taxRate): $tax</p>
                   <p>Total: $total</p>\n";
        }
        ?>
    </body>
</html>
<?php
/**
 * Get a value for parameter in the GET array.
 * @param string $paramName Name of the parameter.
 * @return string|null Value, or null if not found.
 */
function getParamFromGet(string $paramName) {
    $returnValue = null;
    if (isset($_GET[$paramName])) {
        $returnValue = $_GET[$paramName];
        if ($returnValue == '') {
            $returnValue = null;
        }
    }
    return $returnValue;
}

/**
 * Check the state.
 * @param mixed $state State from user.
 * @return string Error message, MT if OK.
 */
function checkState($state) {
    $errorMessage = '';
    $state = strtoupper(trim($state));
    // Is the state known?
    if ($state != 'MI' && $state != 'IL') {
        $errorMessage = "Sorry, '$state' is not a recognized state.<br>";
    }
    // Return results.
    return $errorMessage;
}

/**
 * Check a price.
 * @param mixed $price The price to check.
 * @return string Error message, MT if none.
 */
function checkPrice($price) {
    $priceErrorMessage = '';
    // Check that price is numeric.
    if (!is_numeric($price)) {
        $priceErrorMessage = 'Sorry, price must be a number.<br>';
    }
    // Range check.
    if ($priceErrorMessage == '') {
        if ($price < 0) {
            $priceErrorMessage = 'Sorry, price must be zero or more.<br>';
        }
        if ($price >= 1000000) {
            $priceErrorMessage = 'Sorry, price is too high to be real.<br>';
        }
    }
    return $priceErrorMessage;
}

/**
 * Check a discount rate.
 * @param mixed $discountRate The discount rate to check.
 * @return string Error message, MT if none.
 */
function checkDiscountRate($discountRate) {
    $errorMessage = '';
    // Check that discount rate is numeric.
    if (!is_numeric($discountRate)) {
        $errorMessage = 'Sorry, discount rate must be a number.<br>';
    }
    // Range check.
    if ($errorMessage == '') {
        if ($discountRate < 0) {
            $errorMessage = 'Sorry, discount rate must be zero or more.<br>';
        }
        if ($discountRate >= 0.99) {
            $errorMessage = 'Sorry, discount rate is too high to be real.<br>';
        }
    }
    return $errorMessage;
}
